
##################################################
#  DE Algorithm
#  Carlos Andres Ramirez (email: andres@antai-group.com  URL: https://carlos.engineer/ ) 
#  Last Modified on Oct. 2015
##################################################

import test_func    # C bindings

import sys
import random
import math
import datetime
import time

class MyDE:
    """A simple DE class"""

    def set_params(self, problem_size, pop_size, max_num_evaluations, func_number, test_func) :
        self.dimensions = problem_size
        self.np = pop_size
        self.max_evaluations = max_num_evaluations
        self.function_number = func_number
        self.test_func = 'STEP_PY'          # Default value
        if test_func == 'STEP_C' :
            self.test_func = test_func
        
    def initialize(self):
    
        self.epsilon = 10.0 ** -8
        
        self.cr = 0.90              # Crossover
        self.wf = 0.50              # Weighting factor

        self.f_count = 0            # count

        self.bound_upper = 100
        self.bound_lower = -100
        
        self.func_map = {'STEP_PY' : self.test_function_step_py, 'STEP_C' : self.test_function_c}
        
        # Create the random vectors (Population) and update the objective function (Quality)
        self.Xpopulation = list(map(lambda x: self.generate_random_individual(), range(self.np)))

        self.Xfitness = list(map(lambda i: 0.0, range(0, self.np)))
        self.Vfitness = list(map(lambda i: 0.0, range(0, self.np)))

        self.best_arr = []
        self.best = []                      # Best
        self.best_f = 0                     # Best fitness
        self.init_population_fitness()
        
        self.Vpopulation = []               # Children population
        
    def generate_random_individual(self):
        individual = []
        for i in range(0, self.dimensions) :
            tmp = (( self.bound_upper - self.bound_lower ) * random.random()) + self.bound_lower
            individual.append(tmp)
        return individual

    def bounds_check(self, child, parent) :
        for j in range(0, self.dimensions) :
            if self.Vpopulation[child][j] < self.bound_lower :
                self.Vpopulation[child][j] = (self.bound_lower + self.Xpopulation[parent][j]) / 2.0
            elif self.Vpopulation[child][j] > self.bound_upper:
                self.Vpopulation[child][j] = (self.bound_upper + self.Xpopulation[parent][j]) / 2.0
                       
    def main_loop(self):

        while self.f_count < self.max_evaluations :
        #while self.f_count <= self.max_evaluations and self.best_f > self.epsilon :

            self.Vpopulation = []   # Children list
            for i in range(0, self.np) :
                # TODO: Mutation
                vi = self.mutation_rand_1(i)
                #vi = self.mutation_best_1(i)

                # Generate trial vector ui by crossing xi and vi
                ui = self.crossover_bin(self.Xpopulation[i], vi)
                self.Vpopulation.append(ui)
                self.Vfitness[i] = self.func_map[self.test_func](self.Vpopulation[i])
           
            for i in range(0, self.np) :
                # TODO: < or <=
                if self.Vfitness[i] <= self.Xfitness[i] :
                    self.Xpopulation[i] = self.Vpopulation[i]       # Replace parent
                    self.Xfitness[i] = self.Vfitness[i]
                    if self.Vfitness[i] < self.best_f :             # New best
                         self.best_f = self.Vfitness[i]
                         self.best = self.Vpopulation[i]
                         #tmp = "%d, %d" % (self.f_count, self.best_f)
                         #print tmp

        tmp = "# %d, %d" % (self.f_count, self.best_f)
        print tmp
        #print self.best
        #print self.f_count, ' : ', self.best_f
        
        res = []
        if self.best_f > 0 :
            res.append(1)
        else :
            res.append(self.best_f)
            res.append(self.f_count)
            
        return res

    def prepare_best(self, rand_many = False) :
        self.best_arr = []

        minval = min(self.Xfitness)
        for i in range(0, self.np) :
            if minval == self.Xfitness[i] :
                self.best_arr.append(i)
                if rand_many :
                    return
                    
    def mutation_best_1(self, i) :
        a = self.get_random_population_index()
        while a == i :
            a = self.get_random_population_index()
        
        b = self.get_random_population_index()
        while b == i or b == a :
            b = self.get_random_population_index()

        # TODO: One or many XBest ?
        self.prepare_best(True)
        tmpin = random.randint(0, len(self.best_arr) - 1)
        xbest = self.best_arr[tmpin]
        
        vi = []
        for k in range(0, self.dimensions):
            tmp = self.Xpopulation[xbest][k] + (self.wf * (self.Xpopulation[a][k] - self.Xpopulation[b][k]))
            if tmp < self.bound_lower :
                tmp = (self.bound_lower + self.Xpopulation[i][k]) / 2.0
            elif tmp > self.bound_upper:
                tmp = (self.bound_upper + self.Xpopulation[i][k]) / 2.0
            vi.append(tmp)
        return vi

    def mutation_rand_1(self, i) :
        a = self.get_random_population_index()
        while a == i :
            a = self.get_random_population_index()
        
        b = self.get_random_population_index()
        while b == i or b == a :
            b = self.get_random_population_index()
            
        c = self.get_random_population_index()
        while c == i or c == a or c == b :
            c = self.get_random_population_index()

        vi = []
        for k in range(0, self.dimensions):
            tmp = self.Xpopulation[a][k] + (self.wf * (self.Xpopulation[b][k] - self.Xpopulation[c][k]))
            if tmp < self.bound_lower :
                tmp = (self.bound_lower + self.Xpopulation[i][k]) / 2.0
            elif tmp > self.bound_upper:
                tmp = (self.bound_upper + self.Xpopulation[i][k]) / 2.0
            vi.append(tmp)
        return vi
                        
    def test_function_c(self, vec) :
        tmp = 0
        res = test_func._c_test_func(vec, tmp, self.dimensions, 1, self.function_number)
        self.f_count += 1
        return res

    def test_function_step_py(self, vec) :
        res = 0.0
        
        for i in range(len(vec)) :
            res += 2500 * math.floor(1000 * abs(vec[i]))
            #res += math.floor(vec[i] + 0.5) * math.floor(vec[i] + 0.5)
            
        self.f_count += 1
        return res

    def atest_function_step_py(self, vec) :
        # TODO: Shift
        res = 0.0
        for i in range(len(vec)) :
            res += math.floor(vec[i] + 0.5) * math.floor(vec[i] + 0.5)
        self.f_count += 1
        return res
       
    def get_random_population_index(self):
        return random.randint(0, self.np - 1)
      
    def crossover_bin(self, v1, v2) :
        gene_index = random.randint(0, self.dimensions - 1) + 1
        trial = []
        
        for i in range(0, self.dimensions):
            if random.uniform(0, 1) < self.cr or i == gene_index:
                trial.append(v2[i])
            else :
                trial.append(v1[i])
        return trial

    def init_population_fitness(self):
        self.best_f = self.func_map[self.test_func](self.Xpopulation[0])
        self.Xfitness[0] = self.best_f
        
        for i in range(1, self.np):
             self.Xfitness[i] = self.func_map[self.test_func](self.Xpopulation[i])
             # TODO: < o <=
             if self.Xfitness[i] <= self.best_f :
                 self.best_f = self.Xfitness[i]
                 self.best = self.Xpopulation[i]

##################################################
# Misc procedures
##################################################

def p_out(aStr) :
   	print aStr

def start_main() :
#    problem_size = 20
#    pop_size = 100
#    num_runs = 15

    problem_size = 50
    pop_size = 50
    num_runs = 10

    func_number = 1
    test_func = 'STEP_C'   # 'STEP_PY'

    #max_num_evaluations = (10.0 ** 6) #* 2.0
    max_num_evaluations = 60000
    
    fevals = []
    for i in range(0, num_runs) :
        myde = MyDE()
        myde.set_params(problem_size, pop_size, max_num_evaluations, func_number, test_func)
        myde.initialize()
        res = myde.main_loop()
        if len(res) > 1 :
            fevals.append(res[1])
            
    # print fevals
    
    if len(fevals) >= 1 :
        meanval = sum(fevals)/len(fevals)
        print 'Mean, ', meanval
        print 'Eval, ', meanval/len(fevals)
    else :
        print 'No success.'
##################################################
# Main program
##################################################
p_out('algorithm, Python_DE')
p_out('time_start, ' + str(datetime.datetime.now()))
p_out("##############################")
time_start = time.time()
start_main()
time_end = time.time()
p_out("##############################")
p_out('time_end, ' + str(datetime.datetime.now()))
p_out('time_total, ' + str(time_end - time_start))
##################################################
# EOF
##################################################

