
##################################################
# R language report generation script
#  Carlos Andres Ramirez (email: andres@antai-group.com  URL: https://carlos.engineer/ ) 
#  Last Modified on Oct. 2015
##################################################

import time
import datetime
import sys
import random
import math

class ReportResult:
    """Report generation class."""

    # def __init__(self):

    def set_params(self, total_runs, filepath, outfilepath = '') :
        
        self.num_runs = total_runs

        self.rpath = '#!/usr/bin/Rscript'
        self.filename = filepath
        self.outfilename = outfilepath
        self.raw_data = []
        self.results = []
        self.results_mean = []
        
        self.func_results = []
        self.func_results_mean = []

        self.name = ''
        self.init_time = ''
        self.end_time = ''
        self.total_time = ''

    def get_mean(self, num_func):
        return self.results_mean[num_func -1][0]

    def write_r_script(self, others):
        osize = len(others)
        
        self.rscript = []
        self.rscript.append(self.rpath)
        self.rscript.append("\n\n#####\n")
        tmpstr = "# %s\n" % (self.name.lstrip())
        self.rscript.append(tmpstr)
        self.rscript.append("#####\n")
        self.rscript.append(self.func_results)
        self.rscript.append("\n# Arithmetic mean for all test functions\n")
        self.rscript.append(self.func_results_mean)
        
        for setres in others :
            self.rscript.append("\n#####\n")
            tmpstr = "# %s\n" % (setres.name.lstrip())
            self.rscript.append(tmpstr)
            self.rscript.append("#####\n")
            self.rscript.append(setres.func_results)
            self.rscript.append("\n# Arithmetic mean for all test functions\n")
            self.rscript.append(setres.func_results_mean)
        
        self.rscript.append("\n\n#####\n")
        self.rscript.append("# U-test (Wilcox test)\n")
        self.rscript.append("#####\n")
        
        # U-test (Wilcox test)
        for i in range(0, self.num_functions) :
            for setres in others :
                tmpstr = "wilcox.test(%s, %s, correct = F, paired = F, exact = F, conf.level = 0.95)\n" % (self.name.lstrip() + str(i+1), setres.name.lstrip() + str(i+1))
                self.rscript.append(tmpstr)
            self.rscript.append("\n")
                
        self.write_file(self.rscript)
        

    def generate_r_script_data(self):
        # Test function results: one array per line      
        for func in range(0, self.num_functions) :
            rstring = self.name.lstrip()
            rstring += str(func + 1) + ' = c('
            for run in range(0, self.num_runs) :
                pos = (func * self.num_runs) + run
                rstring += self.results[pos].lstrip()
                if pos == (func * self.num_runs) + self.num_runs - 1 :
                    rstring += ')\n'
                else:
                    rstring += ', '
            #print rstring
            self.func_results.append(rstring)

        # Calculate the mean for each of the test functions
        for item in self.func_results :
            tmp = item.split(' = ')
            tmpstr = tmp[0] + '_mean' + ' = ' + ' mean(' + tmp[0] + ')\n'
            self.func_results_mean.append(tmpstr)

        #self.write_file(self.func_results)
            
    def main_loop(self):
        self.read_raw_file()
        self.results_from_raw()
        
        self.generate_r_script_data()
        #print 'Name: ', self.name
        #print 'init: ', self.init_time
        #print 'end: ', self.end_time
        #print 'total: ', self.total_time
        #print self.get_mean(8)

    def results_from_raw(self):
        if len(self.raw_data) < self.num_functions * self.num_runs :
            print 'results_from_raw: Error. Incomplete data.'
            sys.exit()

        for i in range(0, len(self.raw_data)) :
            if len(self.raw_data[i]) < 1 or self.raw_data[i][0] == '#' :    # Skip empty lines or comments
                continue

            tmp = self.raw_data[i].split(',')
                
            if len(tmp) == 3 and 0 <= int(tmp[0]) <= self.num_runs and 1 <= int(tmp[1]) <= self.num_functions :
                self.results.append(tmp[2])
            elif tmp[0] == 'time_total' :
                self.total_time = tmp[1]
            elif tmp[0] == 'time_end' :
                 self.end_time = tmp[1]
            elif tmp[0] == 'time_start' :
                self.init_time = tmp[1]
            elif tmp[0] == 'algorithm' :
                self.name = tmp[1]
            elif len(tmp) == 2 :    # Mean and Std
                self.results_mean.append(tmp)
            else :
                print 'results_from_raw: Error. Corrupt data.'
                sys.exit()

        if len(self.results) != self.num_functions * self.num_runs :
            print 'results_from_raw: Error. Incomplete results data.'
            sys.exit()
        
    def read_raw_file(self) :
        with open(self.filename, 'r') as file:
            self.raw_data = file.readlines()
        # chomp the newline characters
        for i in range(len(self.raw_data)) :
            self.raw_data[i] = self.raw_data[i].rstrip('\n')

    def write_file(self, data) :
        with open(self.outfilename, 'w') as file:
            for chunk in data :
                file.writelines(chunk)

    def read_ex1(self) :
        with open(self.filename, 'r') as file:
            self.raw_data = file.readlines()

        linea = ''
        linea2 = ''
        anterior = 0
        arrcounter = 1
        ccounter = 0
        # chomp the newline characters
        for i in range(len(self.raw_data)) :
            self.raw_data[i] = self.raw_data[i].rstrip('\n')
            #print self.raw_data[i]
            
            if len(self.raw_data[i]) < 1 or self.raw_data[i][0] == '#' :    # Skip empty lines or comments
                continue

            tmp = self.raw_data[i].split(',')
                
            if tmp[0] == 'time_total' :
                self.total_time = tmp[1]
            elif tmp[0] == 'time_end' :
                 self.end_time = tmp[1]
            elif tmp[0] == 'time_start' :
                self.init_time = tmp[1]
            elif tmp[0] == 'algorithm' :
                self.name = tmp[1]
            elif len(tmp) == 2 :
                current = int(tmp[1])
                if current > anterior :
                    if len(linea) > 0 :
                        linea += ' ) \n'
                        linea2 += ' ) \n'
                        self.results.append(linea)
                        self.results.append(linea2)
                    linea = 'arr_%d <- c( ' % (arrcounter)
                    linea2 = 'arr_%db <- c( ' % (arrcounter)
                    arrcounter += 1
                    ccounter = 0
                else :
                    ccounter += 1
                if ccounter != 0 :
                    linea += ', '
                    linea2 += ', '
                linea += tmp[1]
                linea2 += tmp[0]
                anterior = current
                #print tmp[1]
            else :
                print 'results_from_raw: Error. Corrupt data.'
                sys.exit()


        #a = linea.rstrip(',')
        #b = linea2.rstrip(',')
        
        linea += ' ) \n'
        linea2 += ' ) \n'        
        self.results.append(linea)
        self.results.append(linea2)
        #if len(self.results) != self.num_functions * self.num_runs :
        #    print 'results_from_raw: Error. Incomplete results data.'
        #    sys.exit()
        
        self.write_file(self.results)

                    
##################################################
# Main program procedures
##################################################

def p_out(aStr) :
   	print aStr
        
def start_main() :
    res = ReportResult()
    res.set_params(10, 'less_7.txt', 'r_res.r')
    res.read_ex1()
   
    #res.main_loop()
    
##################################################
# Main program starts
##################################################
start_main()


##################################################
# EOF
##################################################

