
##################################################
# R language report generation script
#  Carlos Andres Ramirez (email: andres@antai-group.com  URL: https://carlos.engineer/ ) 
#  Last Modified on Oct. 2015
##################################################

import time
import datetime
import sys
import random
import math

class ReportResult:
    """Report generation class."""

    # def __init__(self):

    def set_params(self, problem_size, population_size, total_runs, total_functions, filepath, outfilepath = '') :
        
        self.num_runs = total_runs
        self.num_functions = total_functions
        self.dim_size = problem_size
        self.pop_size = population_size

        self.rpath = '#!/usr/bin/Rscript'
        self.filename = filepath
        self.outfilename = outfilepath
        self.raw_data = []
        self.results = []
        self.results_mean = []
        
        self.func_results = []
        self.func_results_mean = []
        self.func_results_std = []

        self.name = ''
        self.init_time = ''
        self.end_time = ''
        self.total_time = ''
        
        # Latex
        self.latex_data = []

    def get_mean(self, num_func):
        return self.results_mean[num_func -1][0]

    def write_r_script(self, others):
        osize = len(others)
        
        self.rscript = []
        self.rscript.append(self.rpath)
        self.rscript.append("\n\n#####\n")
        tmpstr = "# %s\n" % (self.name.lstrip())
        self.rscript.append(tmpstr)
        self.rscript.append("#####\n")
        self.rscript.append(self.func_results)
        self.rscript.append("\n# Arithmetic mean and std for all test functions\n")
        self.rscript.append(self.func_results_mean)
        self.rscript.append(self.func_results_std)
        
        for setres in others :
            self.rscript.append("\n#####\n")
            tmpstr = "# %s\n" % (setres.name.lstrip())
            self.rscript.append(tmpstr)
            self.rscript.append("#####\n")
            self.rscript.append(setres.func_results)
            self.rscript.append("\n# Arithmetic mean for all test functions\n")
            self.rscript.append(setres.func_results_mean)
            self.rscript.append(setres.func_results_std)
        
        self.rscript.append("\n\n#####\n")
        self.rscript.append("# U-test (Wilcox test)\n")
        self.rscript.append("#####\n")
        
        # U-test (Wilcox test)
        for i in range(0, self.num_functions) :

            resname = self.name.lstrip() + str(i+1)
            resname_mean = resname + '_mean'
            resname_std = resname + '_std'

            tmpstr = 'cat(sprintf("%%0.2e", %s), sprintf("%%0.2e", %s), "\\n")\n' % (resname_mean, resname_std)
            self.rscript.append(tmpstr)
            
            for setres in others :
                alternatives = ['two.sided', 'less', 'greater']
                varnames = []
                for alternative in alternatives :
                    mu = 0
                    tmpstr = self.name.lstrip() + str(i+1) + '_' + setres.name.lstrip() + str(i+1)
                    if alternative == 'two.sided' :
                        tmpstr += '_t'
                    if alternative == 'less' :
                        tmpstr += '_l'
                    if alternative == 'greater' :
                        tmpstr += '_g'
                    varnames.append(tmpstr)
                    tmpstr = '%s = wilcox.test(%s, %s, alternative = "%s", mu = %d, conf.level = 0.95, exact = FALSE, correct = FALSE, paired = FALSE)$p.value\n' % (tmpstr, self.name.lstrip() + str(i+1), setres.name.lstrip() + str(i+1), alternative, mu)

                    self.rscript.append(tmpstr)

                tmpstr = '\nif (! is.nan(%s)) {\n' % (varnames[0])
                self.rscript.append(tmpstr)
                
                #for i in range(len(varnames)) :
                resname = setres.name.lstrip() + str(i+1)
                resname_mean = resname + '_mean'
                resname_std = resname + '_std'
                
                tmpstr =  '    if (%s < 0.05) {\n' % (varnames[0])
                tmpstr += '        if (%s <= 0.05) {\n' % (varnames[1])
                tmpstr += '            cat(sprintf("%%0.2e", %s), sprintf("%%0.2e", %s), "%s\\n")\n' % (resname_mean, resname_std, '-') # Soy menor, es decir, el otro es mayor (peor performance)
                tmpstr += '        } else {\n'
                tmpstr += '            cat(sprintf("%%0.2e", %s), sprintf("%%0.2e", %s), "%s\\n")\n' % (resname_mean, resname_std, '+') # Soy mayor, es decir, el otro es menor (mejor performance)
                tmpstr += '        }\n'
                tmpstr += '    } else {\n'
                tmpstr += '        cat(sprintf("%%0.2e", %s), sprintf("%%0.2e", %s), "%s\\n")\n' % (resname_mean, resname_std, '=')
                tmpstr += '    }\n'
                self.rscript.append(tmpstr)
                

                tmpstr = '} else {\n'
                tmpstr += '    cat(sprintf("%%0.2e", %s), sprintf("%%0.2e", %s), "%s\\n")\n' % (resname_mean, resname_std, '=')
                tmpstr += '}\n'
                self.rscript.append(tmpstr)

                self.rscript.append('\n')
                #self.rscript.append('\ncat("***\\n")\n\n')
                    
            self.rscript.append("\n")
                
        self.write_file(self.rscript)
        

    def generate_r_script_data(self):
        # Test function results: one array per line      
        for func in range(0, self.num_functions) :
            rstring = self.name.lstrip()
            rstring += str(func + 1) + ' = c('
            for run in range(0, self.num_runs) :
                pos = (func * self.num_runs) + run
                rstring += self.results[pos].lstrip()
                if pos == (func * self.num_runs) + self.num_runs - 1 :
                    rstring += ')\n'
                else:
                    rstring += ', '
            #print rstring
            self.func_results.append(rstring)

        # Calculate the mean and sd for each of the test functions
        for item in self.func_results :
            tmp = item.split(' = ')
            tmpstr = tmp[0] + '_mean' + ' = ' + ' mean(' + tmp[0] + ')\n'
            self.func_results_mean.append(tmpstr)
            tmpstr = tmp[0] + '_std' + ' = ' + ' sd(' + tmp[0] + ')\n'
            self.func_results_std.append(tmpstr)

        #self.write_file(self.func_results)
            
    def main_loop(self):
        self.read_raw_file()
        self.results_from_raw()
        
        self.generate_r_script_data()
        #print 'Name: ', self.name
        #print 'init: ', self.init_time
        #print 'end: ', self.end_time
        #print 'total: ', self.total_time
        #print self.get_mean(8)

    def results_from_raw(self):
        if len(self.raw_data) < self.num_functions * self.num_runs :
            print 'results_from_raw: Error. Incomplete data.'
            sys.exit()

        for i in range(0, len(self.raw_data)) :
            if len(self.raw_data[i]) < 1 or self.raw_data[i][0] == '#' :    # Skip empty lines or comments
                continue

            tmp = self.raw_data[i].split(',')
                
            if len(tmp) == 3 and 0 <= int(tmp[0]) <= self.num_runs and 1 <= int(tmp[1]) <= self.num_functions :
                self.results.append(tmp[2])
            elif tmp[0] == 'time_total' :
                self.total_time = tmp[1]
            elif tmp[0] == 'time_end' :
                 self.end_time = tmp[1]
            elif tmp[0] == 'time_start' :
                self.init_time = tmp[1]
            elif tmp[0] == 'algorithm' :
                self.name = tmp[1]
            elif len(tmp) == 2 :    # Mean and Std
                self.results_mean.append(tmp)
            else :
                print 'results_from_raw: Error. Corrupt data.'
                sys.exit()

        if len(self.results) != self.num_functions * self.num_runs :
            print 'results_from_raw: Error. Incomplete results data.'
            sys.exit()
        
    def read_raw_file(self) :
        with open(self.filename, 'r') as file:
            self.raw_data = file.readlines()
        # chomp the newline characters
        for i in range(len(self.raw_data)) :
            self.raw_data[i] = self.raw_data[i].rstrip('\n')

    def write_file(self, data) :
        with open(self.outfilename, 'w') as file:
            for chunk in data :
                file.writelines(chunk)
    #####
    # Latex
    #####
    def prepare_latex_data(self, num_cols) :
        routput = 'report_raw.txt'
        with open(routput, 'r') as file:
            raw_data = file.readlines()

        # chomp the newline characters
        for i in range(len(raw_data)) :
            raw_data[i] = raw_data[i].rstrip('\n')

        row = []
        for i in range(len(raw_data)) :
            tmpstr = raw_data[i].split(' ')
            
            if len(tmpstr) == 2 :
                tmp = '%s (%s)' % (tmpstr[0], tmpstr[1])
            elif len(tmpstr) == 3 :
                tmp = '%s (%s)%s' % (tmpstr[0], tmpstr[1], tmpstr[2])
            else :
                print 'prepare_latex_data: Error. Incomplete data or wrong format.'
                sys.exit()

            row.append(tmp)
            
            if (i + 1) % num_cols == 0 :
                self.latex_data.append(row)
                row = []

            #for f in range(0, self.num_functions) :
            
    def print_latex_data(self) :
        for f in range(0, len(self.latex_data)) :
            for i in range(0, len(self.latex_data[f])) :
                tmp = '%s  ' % (self.latex_data[f][i])
                print tmp,
            print "\n"

    def write_latex(self, num_cols) :
        outfile = 'salida.tex'
        with open(outfile, 'w') as file:
            tmpstr = '\\documentclass{book}\n\\begin{document}\n\\vspace{1cm}\n'
            tmp = '\\begin{table}\n\\small\n\\begin{tabular}{'

            for col in range(0, num_cols + 1) :
                tmp += ' c'
            tmp += ' }\n'
            tmpstr += tmp
            file.writelines(tmpstr)
           
            tabledata = '\\hline\n'
            tabledata += 'F & Py Shade & C Shade & Jade & jDE\\\\\n'
            tabledata += '\\hline\n'
            for f in range(0, len(self.latex_data)) :
                tabledata += 'F%d & ' % (f + 1)
                for i in range(0, len(self.latex_data[f])) :
                    if i == len(self.latex_data[f]) - 1:
                        tabledata += '%s \\\\\n' % (self.latex_data[f][i])
                    else :
                        tabledata += '%s & ' % (self.latex_data[f][i])
            tabledata += '\\hline\n'
            
            tabledata += '\\hline\n'
            file.writelines(tabledata)
            tmpstr = '\\end{tabular}\n\\end{table}'
            tmpstr += '\\end{document}\n'
            file.writelines(tmpstr)
            
##################################################
# Main program procedures
##################################################

def p_out(aStr) :
   	print aStr
        
def start_main() :

    de_eq = ReportResult()
    de_eq.set_params(10, 100, 51, 30, '02_D10_LOEQ.txt', '02_D10_BEST1_output.r')
    de_eq.main_loop()

    de_less = ReportResult()
    de_less.set_params(10, 100, 51, 30, '02_D10_BEST1.txt')
    de_less.main_loop()
    
    others = [de_less]
    
    de_eq.write_r_script(others)

    # Latex
    de_eq.prepare_latex_data(len(others) + 1)
    de_eq.print_latex_data()
    de_eq.write_latex(len(others) + 1)
    
    # Automate
    
##################################################
# Main program starts
##################################################
start_main()

# pdflatex ttest.tex
# https://en.wikibooks.org/wiki/LaTeX/Tables

##################################################
# EOF
##################################################

